@@ -78,12 +78,15 @@ struct App
 };
 
 static void rebuild_gui (App *app);
+static void on_clone_changed (GtkWidget *box, gpointer data);
 static void on_rate_changed (GtkComboBox *box, gpointer data);
 static gboolean output_overlaps (GnomeOutputInfo *output, GnomeRRConfig *config);
 static void select_current_output_from_dialog_position (App *app);
 static void monitor_on_off_toggled_cb (GtkToggleButton *toggle, gpointer data);
 static void get_geometry (GnomeOutputInfo *output, int *w, int *h);
 static void apply_configuration_returned_cb (DBusGProxy *proxy, DBusGProxyCall *call_id, void *data);
+static gboolean get_clone_size (GnomeRRScreen *screen, int *width, int *height);
+static gboolean output_info_supports_mode (App *app, GnomeOutputInfo *info, int width, int height);
 
 static void
 error_message (App *app, const char *primary_text, const char *secondary_text)
@@ -118,22 +121,12 @@ idle_free (gchar *s)
     return s;
 }
 
-static int
-compare_outputs (const void *p1, const void *p2)
-{
-    GnomeOutputInfo *const *o1 = p1;
-    GnomeOutputInfo *const *o2 = p2;
-
-    return (**o1).x - (**o2).x;
-}
-
 static void
 on_screen_changed (GnomeRRScreen *scr,
 		   gpointer data)
 {
     GnomeRRConfig *current;
     App *app = data;
-    int i;
 
     current = gnome_rr_config_new_current (app->screen);
 
@@ -143,25 +136,6 @@ on_screen_changed (GnomeRRScreen *scr,
     app->current_configuration = current;
     app->current_output = NULL;
 
-#if 0
-    for (i = 0; app->current_configuration->outputs[i] != NULL; ++i)
-    {
-	GnomeOutputInfo *o = app->current_configuration->outputs[i];
-
-	g_debug ("  output %s %s: %d %d %d %d", o->name, o->on? "on" : "off", o->x, o->y, o->width, o->height);
-    }
-#endif
-
-#if 0
-    g_debug ("sorting");
-#endif
-    /* Sort outputs according to X coordinate */
-    for (i = 0; app->current_configuration->outputs[i] != NULL; ++i)
-	;
-
-    qsort (app->current_configuration->outputs, i, sizeof (GnomeOutputInfo *),
-	   compare_outputs);
-
     if (app->labeler) {
 	gnome_rr_labeler_hide (app->labeler);
 	g_object_unref (app->labeler);
@@ -169,15 +143,6 @@ on_screen_changed (GnomeRRScreen *scr,
 
     app->labeler = gnome_rr_labeler_new (app->current_configuration);
 
-#if 0
-    for (i = 0; app->current_configuration->outputs[i] != NULL; ++i)
-    {
-	GnomeOutputInfo *o = app->current_configuration->outputs[i];
-
-	g_debug ("  output: %d %d %d %d", o->x, o->y, o->width, o->height);
-    }
-#endif
-
     select_current_output_from_dialog_position (app);
 }
 
@@ -458,32 +423,94 @@ count_all_outputs (GnomeRRConfig *config)
 }
 #endif
 
+/* Computes whether "Mirror Screens" (clone mode) is supported based on these criteria:
+ *
+ * 1. There is an available size for cloning.
+ *
+ * 2. There are 2 or more connected outputs that support that size.
+ */
+static gboolean
+mirror_screens_is_supported (App *app)
+{
+    int clone_width, clone_height;
+    gboolean have_clone_size;
+    gboolean mirror_is_supported;
+
+    mirror_is_supported = FALSE;
+
+    have_clone_size = get_clone_size (app->screen, &clone_width, &clone_height);
+
+    if (have_clone_size) {
+	int i;
+	int num_outputs_with_clone_size;
+
+	num_outputs_with_clone_size = 0;
+
+	for (i = 0; app->current_configuration->outputs[i] != NULL; i++)
+	{
+	    GnomeOutputInfo *output = app->current_configuration->outputs[i];
+
+	    /* We count the connected outputs that support the clone size.  It
+	     * doesn't matter if those outputs aren't actually On currently; we
+	     * will turn them on in on_clone_changed().
+	     */
+	    if (output->connected && output_info_supports_mode (app, output, clone_width, clone_height))
+		num_outputs_with_clone_size++;
+	}
+
+	if (num_outputs_with_clone_size >= 2)
+	    mirror_is_supported = TRUE;
+    }
+
+    return mirror_is_supported;
+}
+
+static void
+rebuild_mirror_screens (App *app)
+{
+    gboolean mirror_is_active;
+    gboolean mirror_is_supported;
+
+    g_signal_handlers_block_by_func (app->clone_checkbox, G_CALLBACK (on_clone_changed), app);
+
+    mirror_is_active = app->current_configuration && app->current_configuration->clone;
+
+    /* If mirror_is_active, then it *must* be possible to turn mirroring off */
+    mirror_is_supported = mirror_is_active || mirror_screens_is_supported (app);
+    
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (app->clone_checkbox), mirror_is_active);
+    gtk_widget_set_sensitive (app->clone_checkbox, mirror_is_supported);
+
+    g_signal_handlers_unblock_by_func (app->clone_checkbox, G_CALLBACK (on_clone_changed), app);
+}
+
 static void
 rebuild_current_monitor_label (App *app)
 {
-	char *str;
-	gboolean free_str;
+	char *str, *tmp;
 	GdkColor color;
 	gboolean use_color;
 
 	if (app->current_output)
 	{
-	    str = g_strdup_printf (_("<b>Monitor: %s</b>"), app->current_output->display_name);
-	    free_str = TRUE;
+	    if (app->current_configuration->clone)
+		tmp = g_strdup (_("Mirror Screens"));
+	    else
+		tmp = g_strdup_printf (_("Monitor: %s"), app->current_output->display_name);
+
+	    str = g_strdup_printf ("<b>%s</b>", tmp);
 	    gnome_rr_labeler_get_color_for_output (app->labeler, app->current_output, &color);
 	    use_color = TRUE;
+	    g_free (tmp);
 	}
 	else
 	{
-	    str = _("<b>Monitor</b>");
-	    free_str = FALSE;
+	    str = g_strdup_printf ("<b>%s</b>", _("Monitor"));
 	    use_color = FALSE;
 	}
 
 	gtk_label_set_markup (GTK_LABEL (app->current_monitor_label), str);
-
-	if (free_str)
-	    g_free (str);
+	g_free (str);
 
 	if (use_color)
 	{
@@ -519,7 +546,11 @@ rebuild_on_off_radios (App *app)
     g_signal_handlers_block_by_func (app->monitor_on_radio, G_CALLBACK (monitor_on_off_toggled_cb), app);
     g_signal_handlers_block_by_func (app->monitor_off_radio, G_CALLBACK (monitor_on_off_toggled_cb), app);
 
-    if (app->current_output)
+    sensitive = FALSE;
+    on_active = FALSE;
+    off_active = FALSE;
+
+    if (!app->current_configuration->clone && app->current_output)
     {
 	if (count_active_outputs (app) > 1 || !app->current_output->on)
 	    sensitive = TRUE;
@@ -529,12 +560,6 @@ rebuild_on_off_radios (App *app)
 	on_active = app->current_output->on;
 	off_active = !on_active;
     }
-    else
-    {
-	sensitive = FALSE;
-	on_active = FALSE;
-	off_active = FALSE;
-    }
 
     gtk_widget_set_sensitive (app->monitor_on_radio, sensitive);
     gtk_widget_set_sensitive (app->monitor_off_radio, sensitive);
@@ -552,6 +577,29 @@ make_resolution_string (int width, int height)
     return g_strdup_printf (_("%d x %d"), width, height);
 }
 
+static void
+find_best_mode (GnomeRRMode **modes, int *out_width, int *out_height)
+{
+    int i;
+
+    *out_width = 0;
+    *out_height = 0;
+
+    for (i = 0; modes[i] != NULL; i++)
+    {
+	int w, h;
+
+	w = gnome_rr_mode_get_width (modes[i]);
+	h = gnome_rr_mode_get_height (modes[i]);
+
+	if (w * h > *out_width * *out_height)
+	{
+	    *out_width = w;
+	    *out_height = h;
+	}
+    }
+}
+
 static void
 rebuild_resolution_combo (App *app)
 {
@@ -589,7 +637,12 @@ rebuild_resolution_combo (App *app)
     current = idle_free (make_resolution_string (app->current_output->width, app->current_output->height));
 
     if (!combo_select (app->resolution_combo, current))
-	g_assert_not_reached ();
+    {
+	int best_w, best_h;
+
+	find_best_mode (modes, &best_w, &best_h);
+	combo_select (app->resolution_combo, idle_free (make_resolution_string (best_w, best_h)));
+    }
 }
 
 static void
@@ -610,6 +663,7 @@ rebuild_gui (App *app)
     g_debug ("rebuild gui, is on: %d", app->current_output->on);
 #endif
 
+    rebuild_mirror_screens (app);
     rebuild_current_monitor_label (app);
     rebuild_on_off_radios (app);
     rebuild_resolution_combo (app);
@@ -622,11 +676,6 @@ rebuild_gui (App *app)
     gtk_widget_set_sensitive (app->panel_checkbox, sensitive);
 
     app->ignore_gui_changes = FALSE;
-
-    if (app->current_configuration && app->current_configuration->clone)
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (app->clone_checkbox), TRUE);
-    else
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (app->clone_checkbox), FALSE);
 }
 
 static gboolean
@@ -694,29 +743,6 @@ on_rate_changed (GtkComboBox *box, gpointer data)
     foo_scroll_area_invalidate (FOO_SCROLL_AREA (app->area));
 }
 
-static void
-find_best_mode (GnomeRRMode **modes, int *out_width, int *out_height)
-{
-    int i;
-
-    *out_width = 0;
-    *out_height = 0;
-
-    for (i = 0; modes[i] != NULL; i++)
-    {
-	int w, h;
-
-	w = gnome_rr_mode_get_width (modes[i]);
-	h = gnome_rr_mode_get_height (modes[i]);
-
-	if (w * h > *out_width * *out_height)
-	{
-	    *out_width = w;
-	    *out_height = h;
-	}
-    }
-}
-
 static void
 select_resolution_for_current_output (App *app)
 {
@@ -894,6 +920,69 @@ lay_out_outputs_horizontally (App *app)
 
 }
 
+/* FIXME: this function is copied from gnome-settings-daemon/plugins/xrandr/gsd-xrandr-manager.c.
+ * Do we need to put this function in gnome-desktop for public use?
+ */
+static gboolean
+get_clone_size (GnomeRRScreen *screen, int *width, int *height)
+{
+        GnomeRRMode **modes = gnome_rr_screen_list_clone_modes (screen);
+        int best_w, best_h;
+        int i;
+
+        best_w = 0;
+        best_h = 0;
+
+        for (i = 0; modes[i] != NULL; ++i) {
+                GnomeRRMode *mode = modes[i];
+                int w, h;
+
+                w = gnome_rr_mode_get_width (mode);
+                h = gnome_rr_mode_get_height (mode);
+
+                if (w * h > best_w * best_h) {
+                        best_w = w;
+                        best_h = h;
+                }
+        }
+
+        if (best_w > 0 && best_h > 0) {
+                if (width)
+                        *width = best_w;
+                if (height)
+                        *height = best_h;
+
+                return TRUE;
+        }
+
+        return FALSE;
+}
+
+static gboolean
+output_info_supports_mode (App *app, GnomeOutputInfo *info, int width, int height)
+{
+    GnomeRROutput *output;
+    GnomeRRMode **modes;
+    int i;
+
+    if (!info->connected)
+	return FALSE;
+
+    output = gnome_rr_screen_get_output_by_name (app->screen, info->name);
+    if (!output)
+	return FALSE;
+
+    modes = gnome_rr_output_list_modes (output);
+
+    for (i = 0; modes[i]; i++) {
+	if (gnome_rr_mode_get_width (modes[i]) == width
+	    && gnome_rr_mode_get_height (modes[i]) == height)
+	    return TRUE;
+    }
+
+    return FALSE;
+}
+
 static void
 on_clone_changed (GtkWidget *box, gpointer data)
 {
@@ -905,6 +994,7 @@ on_clone_changed (GtkWidget *box, gpointer data)
     if (app->current_configuration->clone)
     {
 	int i;
+	int width, height;
 
 	for (i = 0; app->current_configuration->outputs[i]; ++i)
 	{
@@ -914,6 +1004,21 @@ on_clone_changed (GtkWidget *box, gpointer data)
 		break;
 	    }
 	}
+
+	/* Turn on all the connected screens that support the best clone mode.
+	 * The user may hit "Mirror Screens", but he shouldn't have to turn on
+	 * all the required outputs as well.
+	 */
+
+	get_clone_size (app->screen, &width, &height);
+
+	for (i = 0; app->current_configuration->outputs[i]; i++) {
+	    if (output_info_supports_mode (app, app->current_configuration->outputs[i], width, height)) {
+		app->current_configuration->outputs[i]->on = TRUE;
+		app->current_configuration->outputs[i]->width = width;
+		app->current_configuration->outputs[i]->height = height;
+	    }
+	}
     }
     else
     {
@@ -937,6 +1042,13 @@ get_geometry (GnomeOutputInfo *output, int *w, int *h)
 	*h = output->pref_height;
 	*w = output->pref_width;
     }
+   if ((output->rotation & GNOME_RR_ROTATION_90) || (output->rotation & GNOME_RR_ROTATION_270))
+   {
+        int tmp;
+        tmp = *h;
+        *h = *w;
+        *w = tmp;
+   }
 }
 
 #define SPACE 15
@@ -1541,11 +1653,12 @@ paint_output (App *app, cairo_t *cr, int i)
     GList *connected_outputs = list_connected_outputs (app, &total_w, &total_h);
     GnomeOutputInfo *output = g_list_nth (connected_outputs, i)->data;
     PangoLayout *layout = get_display_name (app, output);
-    PangoRectangle extent;
+    PangoRectangle ink_extent, log_extent;
     GdkRectangle viewport;
-    double angle;
     GdkColor output_color;
     double r, g, b;
+    double available_w;
+    double factor;
 
     cairo_save (cr);
 
@@ -1578,28 +1691,7 @@ paint_output (App *app, cairo_t *cr, int i)
 		     x + (w * scale + 0.5) / 2,
 		     y + (h * scale + 0.5) / 2);
 
-    if (output->rotation & GNOME_RR_ROTATION_0)
-    {
-	angle = 0;
-    }
-    else if (output->rotation & GNOME_RR_ROTATION_90)
-    {
-	angle = 1.5 * G_PI;
-    }
-    else if (output->rotation & GNOME_RR_ROTATION_180)
-    {
-	angle = G_PI;
-    }
-    else if (output->rotation & GNOME_RR_ROTATION_270)
-    {
-	angle = G_PI / 2;
-    }
-    else
-    {
-	angle = 0;
-    }
-
-    cairo_rotate (cr, angle);
+    /* rotation is already applied in get_geometry */
 
     if (output->rotation & GNOME_RR_REFLECT_X)
 	cairo_scale (cr, -1, 1);
@@ -1652,13 +1744,19 @@ paint_output (App *app, cairo_t *cr, int i)
     cairo_set_line_width (cr, 2);
 
     layout_set_font (layout, "Sans Bold 12");
+    pango_layout_get_pixel_extents (layout, &ink_extent, &log_extent);
 
-    pango_layout_get_pixel_extents (layout, NULL, &extent);
+    available_w = w * scale + 0.5 - 6; /* Same as the inner rectangle's width, minus 1 pixel of padding on each side */
+    if (available_w < ink_extent.width)
+	factor = available_w / ink_extent.width;
+    else
+	factor = 1.0;
 
-    extent.x = x + ((w * scale + 0.5) - extent.width) / 2;
-    extent.y = y + ((h * scale + 0.5) - extent.height) / 2;
+    cairo_move_to (cr,
+		   x + ((w * scale + 0.5) - factor * log_extent.width) / 2,
+		   y + ((h * scale + 0.5) - factor * log_extent.height) / 2);
 
-    cairo_move_to (cr, extent.x, extent.y);
+    cairo_scale (cr, factor, factor);
 
     if (output->on)
 	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
