@@ -73,13 +73,13 @@ draw_string(Display *xdisplay, XftDraw *draw, XftFont *font, XftColor *colour,
     XGlyphInfo extents;
     gint len = strlen(text);
 
-    XftTextExtentsUtf8(xdisplay, font, text, len, &extents);
-    XftDrawStringUtf8(draw, colour, font, 4, *pos_y + extents.y, text, len);
+    XftTextExtentsUtf8(xdisplay, font, (guchar *)text, len, &extents);
+    XftDrawStringUtf8(draw, colour, font, 4, *pos_y + extents.y, (guchar *)text, len);
     *pos_y += extents.height + 4;
 }
 
 static gboolean
-check_font_contain_text (FT_Face face, gchar *text)
+check_font_contain_text (FT_Face face, const gchar *text)
 {
     while (text && *text)
 	{
@@ -98,7 +98,7 @@ create_text_pixmap(GtkWidget *drawing_area, FT_Face face)
 {
     gint i, pixmap_width, pixmap_height, pos_y, textlen;
     GdkPixmap *pixmap = NULL;
-    gchar *text;
+    const gchar *text;
     Display *xdisplay;
     Drawable xdrawable;
     Visual *xvisual;
@@ -110,10 +110,10 @@ create_text_pixmap(GtkWidget *drawing_area, FT_Face face)
     gint *sizes = NULL, n_sizes, alpha_size;
     FcCharSet *charset = NULL;
 
-    text = _("The quick brown fox jumps over the lazy dog. 0123456789");
+    text = pango_language_get_sample_string(NULL);
     if (! check_font_contain_text (face, text))
 	{
-	    text = "The quick brown fox jumps over the lazy dog. 0123456789";
+	    pango_language_get_sample_string (pango_language_from_string ("en_US"));
 	}
 
     textlen = strlen(text);
@@ -161,15 +161,15 @@ create_text_pixmap(GtkWidget *drawing_area, FT_Face face)
     font = get_font(xdisplay, face, alpha_size, charset);
     charset = FcCharSetCopy (font->charset);
     XftTextExtentsUtf8(xdisplay, font,
-		       lowercase_text, strlen(lowercase_text), &extents);
+		       (guchar *)lowercase_text, strlen(lowercase_text), &extents);
     pixmap_height += extents.height + 4;
     pixmap_width = MAX(pixmap_width, 8 + extents.width);
     XftTextExtentsUtf8(xdisplay, font,
-		       uppercase_text, strlen(uppercase_text), &extents);
+		       (guchar *)uppercase_text, strlen(uppercase_text), &extents);
     pixmap_height += extents.height + 4;
     pixmap_width = MAX(pixmap_width, 8 + extents.width);
     XftTextExtentsUtf8(xdisplay, font,
-		       punctuation_text, strlen(punctuation_text), &extents);
+		       (guchar *)punctuation_text, strlen(punctuation_text), &extents);
     pixmap_height += extents.height + 4;
     pixmap_width = MAX(pixmap_width, 8 + extents.width);
     XftFontClose(xdisplay, font);
@@ -179,7 +179,7 @@ create_text_pixmap(GtkWidget *drawing_area, FT_Face face)
     for (i = 0; i < n_sizes; i++) {
 	font = get_font(xdisplay, face, sizes[i], charset);
 	if (!font) continue;
-	XftTextExtentsUtf8(xdisplay, font, text, textlen, &extents);
+	XftTextExtentsUtf8(xdisplay, font, (guchar *)text, textlen, &extents);
 	pixmap_height += extents.height + 4;
 	pixmap_width = MAX(pixmap_width, 8 + extents.width);
 	XftFontClose(xdisplay, font);
@@ -332,17 +332,17 @@ add_face_info(GtkWidget *table, gint *row_p, const gchar *uri, FT_Face face)
 	    switch (sname.name_id) {
 	    case TT_NAME_ID_COPYRIGHT:
 		g_free(copyright);
-		copyright = g_convert(sname.string, sname.string_len,
+		copyright = g_convert((gchar *)sname.string, sname.string_len,
 				      "UTF-8", "UTF-16BE", NULL, NULL, NULL);
 		break;
 	    case TT_NAME_ID_VERSION_STRING:
 		g_free(version);
-		version = g_convert(sname.string, sname.string_len,
+		version = g_convert((gchar *)sname.string, sname.string_len,
 				    "UTF-8", "UTF-16BE", NULL, NULL, NULL);
 		break;
 	    case TT_NAME_ID_DESCRIPTION:
 		g_free(description);
-		description = g_convert(sname.string, sname.string_len,
+		description = g_convert((gchar *)sname.string, sname.string_len,
 					"UTF-8", "UTF-16BE", NULL, NULL, NULL);
 		break;
 	    default:
@@ -354,7 +354,7 @@ add_face_info(GtkWidget *table, gint *row_p, const gchar *uri, FT_Face face)
 	    g_free(version);
 	}
 	if (copyright) {
-	    add_row(table, row_p, _("Copyright:"), copyright, TRUE, FALSE);
+	    add_row(table, row_p, _("Copyright:"), copyright, TRUE, TRUE);
 	    g_free(copyright);
 	}
 	if (description) {
@@ -474,14 +474,14 @@ install_button_clicked_cb (GtkButton   *button,
     g_free (dest_path);
 
     /* create destination filename */
-    dest_filename = g_path_get_basename (font_file);
+    src = g_file_new_for_uri (font_file);
+
+    dest_filename = g_file_get_basename (src);
     dest_path = g_build_filename (g_get_home_dir (), ".fonts", dest_filename, NULL);
     g_free (dest_filename);
 
     dest = g_file_new_for_path (dest_path);
 
-    src = g_file_new_for_uri (font_file);
-
     /* TODO: show error dialog if file exists */
     g_file_copy_async (src, dest, G_FILE_COPY_NONE, 0, NULL, NULL, NULL,
                        font_install_finished_cb, button);
